def parse_markdown(markdown_text: str) -> list:
    """Parse le Markdown enrichi et retourne l'AST."""


def ast_to_mjml(ast: list) -> str:
    """Transforme l'AST en MJML."""


def markdown_to_mjml(markdown_text: str) -> str:
    """Pipeline complet: Markdown -> AST -> MJML."""
